<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/gj-logo.png', 1) }}" title="Developed by Grant Jenks" width="200"/>
  </a>
</p>

<p>
<iframe src="http://ghbtns.com/github-btn.html?user=grantjenks&repo=sorted_containers&type=watch&count=true&size=large"
  allowtransparency="true" frameborder="0" scrolling="0" width="200px" height="35px"></iframe>
</p>

<p>
  SortedContainers provides sorted container types, written in
  pure-Python and fast as C-extensions.
</p>

<h3>Give Support</h3>
<p>If you or your organization uses SortedContainers, consider financial support:</p>
<p>
  <a href="http://gum.co/sortedcontainers" target="_blank">Give to Sorted Containers</a>
</p>

<h3>Useful Links</h3>
<ul>
  <li><a href="http://www.grantjenks.com/docs/sortedcontainers/">SortedContainers Module</a></li>
  <li><a href="https://pypi.python.org/pypi/sortedcontainers">SortedContainers @ PyPI</a></li>
  <li><a href="https://github.com/grantjenks/sorted_containers">SortedContainers @ Github</a></li>
  <li><a href="https://github.com/grantjenks/sorted_containers/issues">Issue Tracker</a></li>
</ul>
